UNIT HIDform;

INTERFACE

USES
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls,
{$IFDEF USE_DLL}
  HIDctrlIntf;
{$ELSE}
  HIDinterface;
{$ENDIF}

TYPE
  THIDtestForm = class(TForm)
    ScanBut: TButton;
    InputWindow: TMemo;
    OpenBut: TButton;
    NumberBox: TEdit;
    OpenIndicator: TShape;
    SendBut: TButton;
    OutputWindow: TMemo;
    ClearSendBut: TButton;
    ClearRecBut: TButton;
    Label1: TLabel;
    VID: TEdit;
    PID: TEdit;
    Label2: TLabel;
    TargetSelector: TCheckBox;
    HexSelector: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    procedure ScanButClick(Sender: TObject);
    procedure OpenButClick(Sender: TObject);
    procedure SendButClick(Sender: TObject);
    procedure ClearRecButClick(Sender: TObject);
    procedure ClearSendButClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure VIDChange(Sender: TObject);
    procedure TargetSelectorClick(Sender: TObject);
    procedure PIDChange(Sender: TObject);
    procedure BootLoaderButClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    Procedure SaveSettings;
    Procedure NewInit;
    Procedure LoadSettings;
  public
    { Public declarations }
    Procedure ShowDeviceData( Nr     : Integer;
                              Device : THIDdeviceInfo);
  end;

TYPE
  TSetup        = Record
                    Xm, Ym     : Integer;
                    Xb, Yb     : Integer;
                    Vid        : Word;
                    Pid        : Word;
                    BootOffset : Cardinal;
                    MakeFiles  : Boolean;
                    SerialNum  : Integer;
                    LastDirR   : String[255];
                    LastFileR  : String[255];
                    LastDirS   : String[255];
                    LastFileS   : String[255];
                  end;

VAR
  HIDtestForm    : THIDtestForm;
  BootLoaderMode : Boolean = FALSE;
  ProgDir        : String;
//  TargetVID      : Word    = 0;  // equals to any device
//  TargetPID      : Word    = 0;
//  BootInit       : Cardinal = $A0EF32F0;// first 4 bytes of bootloader image
  Setup, pSetup  : TSetup;

Procedure HIDreader( Data : THIDbuffer); stdcall;

IMPLEMENTATION

{$IFDEF USE_BOOTLOADER}
USES
  BootLoad;
{$ENDIF}

VAR
  DevList       : THIDdeviceList;
  IsOpen        : Boolean = FALSE;
  CurrentDevice : Integer = -1;
  FirstInit     : Boolean = TRUE;

{$R *.dfm}

Function SomethingChanged : Boolean;
Begin
  Result :=
            (Setup.Xm           < pSetup.Xm - 2) OR
            (Setup.Xm           > pSetup.Xm + 2) OR
            (Setup.Ym           < pSetup.Ym - 2) OR
            (Setup.Ym           > pSetup.Ym + 2) OR
            (Setup.Xb           < pSetup.Xb - 2) OR
            (Setup.Xb           > pSetup.Xb + 2) OR
            (Setup.Yb           < pSetup.Yb - 2) OR
            (Setup.Yb           > pSetup.Yb + 2) OR
            (Setup.Xb          <> pSetup.Xb)         OR
            (Setup.Yb          <> pSetup.Yb)         OR
            (Setup.Vid         <> pSetup.Vid)        OR
            (Setup.Pid         <> pSetup.Pid)        OR
            (Setup.BootOffset  <> pSetup.BootOffset) OR
            (Setup.MakeFiles   <> pSetup.MakeFiles)  OR
            (Setup.SerialNum   <> pSetup.SerialNum)  OR
            (Setup.LastDirR    <> pSetup.LastDirR)   OR
            (Setup.LastFileR   <> pSetup.LastFileR)  OR
            (Setup.LastDirS    <> pSetup.LastDirS)   OR
            (Setup.LastFileS   <> pSetup.LastFileS);

End;

Procedure THIDtestForm.SaveSettings;
Var
  F : File of TSetup;
Begin
  With Setup do
    begin
      Xm          := HIDtestForm.Left;
      Ym          := HIDtestForm.Top;
    end;
  If SomethingChanged then
    If (MessageDlg('Closing program... Save settings?',
                     mtInformation, [mbYes, mbNo], 0) = mrYes) then
      begin
        AssignFile(F, ProgDir + 'HidTest.dat');
        Rewrite(F);
        Write(F, Setup);
        CloseFile(F);
      end;
End;

Procedure THIDtestForm.NewInit;
Begin
  // default values point to mE bootloader specification
  Setup.Vid := $1234;
  Setup.Pid := $0001;
  Setup.BootOffset  := $A0EF32F0;
  Setup.MakeFiles := FALSE;
  Setup.SerialNum := 1;
  Setup.Xm := (Screen.Width  DIV 2) - (HIDtestForm.Width  DIV 2);
  Setup.Ym := (Screen.Height DIV 2) - (HIDtestForm.Height DIV 2);
End;

Procedure THIDtestForm.LoadSettings;
Var
  F : File of TSetup;
Begin
  AssignFile(F, ProgDir + 'HidTest.dat');
  {$I-}
  Reset(F);
  If (IORESULT = 0) then
    begin
      Read(F, Setup);
      If (IORESULT = 0) then
        With Setup do
          begin
            HIDtestForm.Left    := Xm;
            HIDtestForm.Top     := Ym;
          end
      else NewInit;
      CloseFile(F);
    end
  else NewInit;
  pSetup := Setup;
End;


Procedure THIDtestForm.ShowDeviceData( Nr     : Integer;
                                       Device : THIDdeviceInfo);
Begin
  With Device do
    begin
      InputWindow.Lines.Add('Device Number : '   + IntToStr(Nr));
      InputWindow.Lines.Add('Symbolic link : '   + SymLink);
      InputWindow.Lines.Add('Handle        : 0x' + IntToHex(Handle, 1));
      InputWindow.Lines.Add('Buffer size   : '   + IntToStr(BufferSize));
      InputWindow.Lines.Add('VID           : 0x' + IntToHex(VID, 4));
      InputWindow.Lines.Add('PID           : 0x' + IntToHex(PID, 4));
      InputWindow.Lines.Add('Version       : '   + IntToStr(HI(VersionNumber)) + '.' +
                                                   IntToStr(LO(VersionNumber)));
      InputWindow.Lines.Add('Manufacturer  : '   + ManufacturerString);
      InputWindow.Lines.Add('Product name  : '   + ProductString);
      InputWindow.Lines.Add('Serial number : '   + SerialNumberString);
      InputWindow.Lines.Add(' ');
    end;
End;

procedure THIDtestForm.ScanButClick(Sender: TObject);
Var
  NumDevices, I :Byte;
begin
  If IsOpen then HIDcloseDevice( DevList[ CurrentDevice]);
  OpenIndicator.Brush.Color := clBtnFace;
  HIDscanForDevices( DevList,
                     NumDevices,
                     Setup.VID,
                     Setup.PID);
  If (NumDevices > 0) then
    begin
      InputWindow.Lines.Add(IntToStr(NumDevices) + ' device(s) found');
      For I := 0 to NumDevices - 1 do ShowDeviceData(I, DevList[I]);
    end
  else InputWindow.Lines.Add('No devices found');
  InputWindow.SetFocus;
end;

procedure THIDtestForm.OpenButClick(Sender: TObject);
begin
  If (TRIM(NumberBox.Text) = '') then Exit;
  If IsOpen then HIDcloseDevice( DevList[ CurrentDevice]);
  OpenIndicator.Brush.Color := clBtnFace;
  TRY
    CurrentDevice := StrToInt(NumberBox.Text);
  EXCEPT
    Exit;
  END;
  IsOpen := FALSE;
  If HIDopenDevice( DevList[ CurrentDevice]) then
    begin
      OpenIndicator.Brush.Color := clLime;
      IsOpen := TRUE;
      ShowDeviceData( CurrentDevice,
                      DevList[ CurrentDevice]);
    end
  else OpenIndicator.Brush.Color := clRed;
  OutputWindow.SetFocus;
end;

Procedure HIDreader( Data : THIDbuffer); stdcall;
// Callback procedure to capture incoming data
// data is put into a String for representation
Var
  InStr,
  HexStr : AnsiString;
  I      : Integer;
Begin
  InStr := '';
  For I := 0 to SizeOf(THIDbuffer) - 1 do InStr := InStr + CHR(Data[I]);
  HIDtestForm.InputWindow.Lines.Add(InStr);
  If HIDtestForm.HexSelector.Checked then
    begin
      HexStr := '';
      For I := 1 to Length(InStr) do HexStr := HexStr + IntToHex(ORD(InStr[I]), 2) + ' ';
      HIDtestForm.InputWindow.Lines.Add('HEX:');
      HIDtestForm.InputWindow.Lines.Add(HexStr);
      HIDtestForm.InputWindow.Lines.Add('End HEX');
    end;
End;

Procedure CheckUSBevent;  stdcall;
// Callback procedure for USB change event
Begin
{$IFDEF USE_BOOTLOADER}
  If BootLoaderMode then CheckLoader
  else
    begin
      HIDtestForm.InputWindow.Lines.Add('USB event');
      HIDtestForm.ScanButClick(nil);
    end;
{$ELSE}
  HIDtestForm.InputWindow.Lines.Add('USB event');
  HIDtestForm.ScanButClick(nil);
{$ENDIF}
End;

procedure THIDtestForm.SendButClick(Sender: TObject);
Var
  SendStr  : AnsiString;
  Buffer   : THIDbuffer;
  Cnt      : Byte;
  OK       : Boolean;
begin
  If NOT IsOpen then Exit;  // do not send data to un-opened device
  SendStr := OutputWindow.Text;
  OK      := TRUE;
  While (Length(SendStr) > 0) AND OK do
    begin
      For Cnt := 0 to 63 do
        If (Cnt < Length(SendStr)) then Buffer[Cnt] := ORD(SendStr[Cnt + 1])
        else Buffer[Cnt] := 0;
      OK := HIDwriteDevice( DevList[CurrentDevice],
                            Buffer);
      Delete(SendStr, 1, 64);
      Application.ProcessMessages;
    end;
  If NOT OK then OpenIndicator.Brush.Color := clred;
end;

procedure THIDtestForm.ClearRecButClick(Sender: TObject);
begin
  InputWindow.Lines.Clear;
end;

procedure THIDtestForm.ClearSendButClick(Sender: TObject);
begin
  OutputWindow.Lines.Clear;
end;

procedure THIDtestForm.TargetSelectorClick(Sender: TObject);
begin
  With (Sender as TCheckbox) do
    If Checked then
      begin
        Setup.VID := StrToInt('$' + VID.Text);
        Setup.PID := StrToInt('$' + PID.Text);
      end
    else
      begin
        Setup.VID := 0;
        Setup.PID := 0;
      end
end;

procedure THIDtestForm.VIDChange(Sender: TObject);
begin
  With (Sender as TEdit) do
    If (TRIM(Text) = '') then Text := '0'
    else
      begin
        TRY
          Setup.VID := StrToInt('$' + Text);
        EXCEPT
          Text := '0';
        END;
      end;
end;

procedure THIDtestForm.PIDChange(Sender: TObject);
begin
  With (Sender as TEdit) do
    If (TRIM(Text) = '') then Text := '0'
    else
      begin
        TRY
          Setup.PID := StrToInt('$' + Text);
        EXCEPT
          Text := '0';
        END;
      end;
end;

procedure THIDtestForm.BootLoaderButClick(Sender: TObject);
begin
{$IFDEF USE_BOOTLOADER}
  BootLoaderMode := TRUE;
  BootloaderForm.ShowModal;
{$ENDIF}
end;

procedure THIDtestForm.FormCreate(Sender: TObject);
begin
{$IFDEF USE_BOOTLOADER}
  BootLoaderBut.Visible := TRUE;
{$ENDIF}
  HIDsetEventHandler( @HIDreader);  // assign callback procedure for HID reader
  USBsetEventHandler( @CheckUSBevent);  // assign callback procedure for USB change event
end;

procedure THIDtestForm.FormActivate(Sender: TObject);
begin
  If FirstInit then
    begin
      FirstInit := FALSE;
      LoadSettings;
    end;

end;

procedure THIDtestForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  SaveSettings;
end;

INITIALIZATION
  ProgDir  := ExtractFilepath(Application.Exename);
  If ProgDir[Length(ProgDir)] <> '\'
    then ProgDir := ProgDir + '\';
  ChDir(ProgDir);
END.


