unit HIDform;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls,
  HID_Intf
  ;

type
  THIDtestForm = class(TForm)
    ScanBut: TButton;
    InputWindow: TMemo;
    OpenBut: TButton;
    NumberBox: TEdit;
    OpenIndicator: TShape;
    OutputWindow: TMemo;
    ClearRecButton: TButton;
    HexSelector: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    CloseButton: TButton;
    ReportIdEdit: TEdit;
    Label1: TLabel;
    GetReportButton: TButton;
    SetReportButton: TButton;
    GetFeatureButton: TButton;
    SetFeatureButton: TButton;
    SendButton: TButton;
    ClearSendButton: TButton;
    procedure ScanButClick(Sender: TObject);
    procedure OpenButClick(Sender: TObject);
    procedure SendButtonClick(Sender: TObject);
    procedure ClearRecButtonClick(Sender: TObject);
    procedure ClearSendButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure GetReportButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ShowDeviceData(Nr: Integer;
      Device: THIDdeviceInfo);
    procedure UpdateKeys;
  end;

var
  HIDtestForm: THIDtestForm;
  ProgDir: string;

implementation

var
  IsOpen: Boolean = FALSE;
  CurrentDevice: integer = -1;
  NumDevices: byte;

{$R *.dfm}

procedure THIDtestForm.UpdateKeys;
begin
  CloseButton.Enabled := IsOpen;

  SendButton.Enabled := IsOpen and (HIDOutputReportSize > 0);
  SetReportButton.Enabled := SendButton.Enabled;
  GetReportButton.Enabled := IsOpen and (HIDInputReportSize > 0);

  SetfeatureButton.Enabled := IsOpen and (HIDFeatureReportSize > 0);
  GetfeatureButton.Enabled := SetfeatureButton.Enabled;
end;

procedure THIDtestForm.ShowDeviceData(Nr: Integer;
  Device: THIDdeviceInfo);
var
  I: byte;
  TmpS: string;
begin
  with Device do
  begin
    InputWindow.Lines.Add('');
    InputWindow.Lines.Add('------------------------------------------------');
    InputWindow.Lines.Add('');

    InputWindow.Lines.Add('Device Number       : ' + IntToStr(Nr));
    InputWindow.Lines.Add('');
    TmpS := ManufacturerString + ' ' + ProductString + ' VID:' + IntToHex(VID, 4) + ' PID:' + IntToHex(PID, 4);

    if HasReadWriteAccess
      then TmpS := TmpS + ' R/W'
    else TmpS := TmpS + ' R/O';
    InputWindow.Lines.Add(TmpS);

    InputWindow.Lines.Add('');
    InputWindow.Lines.Add('Output Buffer size  : ' + IntToStr(OutputBufferSize));
    InputWindow.Lines.Add('Input Buffer size   : ' + IntToStr(InputBufferSize));
    InputWindow.Lines.Add('Feature Buffer size : ' + IntToStr(FeatureBufferSize));

      // Display the output report's capabilities
    TmpS := '';
    for I := 0 to MaxNrCapabilities - 1 do
      if OutputReportCaps[I].ReportCount > 0 then
        TmpS :=
          TmpS + 'ReportId ' + IntToStr(OutputReportCaps[I].ReportId) {+ ', Usage ' + IntToStr(OutputReportCaps[I].Usage)} +
          ' (' + IntToStr(OutputReportCaps[I].ReportCount) + ' bytes); '; {' times ' + IntToStr(OutputReportCaps[I].ReportSize) + ' bits); ';}

    if TmpS = '' then TmpS := '0';
    TmpS := trim(TmpS);
    if TmpS[length(TmpS)] = ';' then delete(TmpS, length(TmpS), 1);
    InputWindow.Lines.Add('Output Report(s)    : ' + TmpS);

      // Display the input report's capabilities
    TmpS := '';
    for I := 0 to MaxNrCapabilities - 1 do
      if InputReportCaps[I].ReportCount > 0 then
        TmpS := TmpS + 'ReportId ' + IntToStr(InputReportCaps[I].ReportId) {+ ', Usage ' + IntToStr(InputReportCaps[I].Usage)} +
          ' (' + IntToStr(InputReportCaps[I].ReportCount) + ' bytes); '; { times ' + IntToStr(InputReportCaps[I].ReportSize) + ' bits); '; }

    if TmpS = '' then TmpS := '0';
    TmpS := trim(TmpS);
    if TmpS[length(TmpS)] = ';' then delete(TmpS, length(TmpS), 1);
    InputWindow.Lines.Add('Input Report(s)     : ' + TmpS);

      // Display the feature report's capabilities
    TmpS := '';
    for I := 0 to MaxNrCapabilities - 1 do
      if FeatureReportCaps[I].ReportCount > 0 then
        TmpS := TmpS + 'ReportId ' + IntToStr(FeatureReportCaps[I].ReportId) {+ ', Usage ' + IntToStr(FeatureReportCaps[I].Usage)} +
          ' (' + IntToStr(FeatureReportCaps[I].ReportCount) + ' bytes); '; { times ' + IntToStr(FeatureReportCaps[I].ReportSize) + ' bits); '; }

    if TmpS = '' then TmpS := '0';
    TmpS := trim(TmpS);
    if TmpS[length(TmpS)] = ';' then delete(TmpS, length(TmpS), 1);
    InputWindow.Lines.Add('Feature Report(s)   : ' + TmpS);

  end;
end;

procedure THIDtestForm.ScanButClick(Sender: TObject);
var
  I: Byte;
  Info: THIDDeviceInfo;
begin

  if IsOpen then HIDcloseDevice;
  IsOpen := false;

  OpenIndicator.Brush.Color := clBtnFace;

  NumDevices := HIDscanForDevices;

  if (NumDevices > 0) then
  begin
    InputWindow.Lines.Add(IntToStr(NumDevices) + ' device(s) found');
    for I := 0 to NumDevices - 1 do
    begin
      HIDGetDeviceInfo(I, Info);
      ShowDeviceData(I, Info);
    end;
  end
  else InputWindow.Lines.Add('No devices found');

  InputWindow.SetFocus;
  UpdateKeys;
end;

procedure THIDtestForm.OpenButClick(Sender: TObject);
var
  Info: THIDDeviceInfo;
begin
  if NumDevices = 0 then exit;

  if (TRIM(NumberBox.Text) = '') then Exit;
  if IsOpen then HIDcloseDevice;

  OpenIndicator.Brush.Color := clBtnFace;
  try
    CurrentDevice := StrToInt(NumberBox.Text);
  except
    Exit;
  end;
  IsOpen := FALSE;

  if HIDopenDevice(CurrentDevice) then
  begin
    OpenIndicator.Brush.Color := clLime;
    IsOpen := TRUE;
    HIDGetDeviceInfo(CurrentDevice, Info);
    ShowDeviceData(CurrentDevice, Info);
  end
  else
  begin
    OpenIndicator.Brush.Color := clRed;
    CurrentDevice := -1;
  end;

  OutputWindow.SetFocus;
  UpdateKeys;
end;

procedure HIDreader(Report: THIDReport; NumBytes: byte); stdcall;
// Callback procedure to capture incoming data
// data is put into a String for representation
var
  InStr, HexStr: AnsiString;
  I: Integer;
begin
  HIDtestForm.InputWindow.Lines.Add('');
  HIDtestForm.InputWindow.Lines.Add('Received');

  HIDtestForm.InputWindow.Lines.Add('ReportId: ' + IntToStr(Report.ReportID));

  InStr := '';
  for I := 0 to NumBytes - 1 do InStr := InStr + CHR(Report.Data[I]);
  HIDtestForm.InputWindow.Lines.Add(InStr);

  if HIDtestForm.HexSelector.Checked then
  begin
    HexStr := '';
    for I := 1 to Length(InStr) do HexStr := HexStr + IntToHex(ORD(InStr[I]), 2) + ' ';
    HIDtestForm.InputWindow.Lines.Add('HEX:');
    HIDtestForm.InputWindow.Lines.Add(HexStr);
    HIDtestForm.InputWindow.Lines.Add('End HEX');
  end;
  HIDtestForm.InputWindow.Lines.Add('');
end;

procedure THIDtestForm.GetReportButtonClick(Sender: TObject);
var
  InStr, HexStr: AnsiString;
  I: Integer;
  Report: THIDReport;
  NumBytes: byte;
begin
  Report.ReportID := StrToInt(ReportIdEdit.Text);

  if sender = GetReportButton
    then NumBytes := HIDGetInputReport(Report)
  else NumBytes := HIDGetFeatureReport(Report);

  if NumBytes = 0 then exit;

  HIDtestForm.InputWindow.Lines.Add('');
  if sender = GetReportButton then HIDtestForm.InputWindow.Lines.Add('Get Report:');
  if sender = GetFeatureButton then HIDtestForm.InputWindow.Lines.Add('Get Feature:');

  HIDtestForm.InputWindow.Lines.Add('ReportId: ' + IntToStr(Report.ReportID));

  InStr := '';
  for I := 0 to NumBytes - 1 do InStr := InStr + CHR(Report.Data[I]);
  HIDtestForm.InputWindow.Lines.Add(InStr);

  if HIDtestForm.HexSelector.Checked then
  begin
    HexStr := '';
    for I := 1 to Length(InStr) do HexStr := HexStr + IntToHex(ORD(InStr[I]), 2) + ' ';
    HIDtestForm.InputWindow.Lines.Add('HEX:');
    HIDtestForm.InputWindow.Lines.Add(HexStr);
    HIDtestForm.InputWindow.Lines.Add('End HEX');
  end;
  HIDtestForm.InputWindow.Lines.Add('');
end;

procedure CheckUSBevent; stdcall;
// Callback procedure for USB change event
begin
  HIDtestForm.InputWindow.Lines.Add('');
  HIDtestForm.InputWindow.Lines.Add('USB event');
  HIDtestForm.InputWindow.Lines.Add('');
  HIDtestForm.ScanButClick(nil);
end;

procedure THIDtestForm.SendButtonClick(Sender: TObject);
var
  SendStr: AnsiString;
  Report: THIDReport;
  Cnt: Byte;
  NumBytes: byte;
  OK: boolean;
begin
  if not IsOpen then Exit; // do not send data to un-opened device

  SendStr := trim(OutputWindow.Text);
  OK := TRUE;

  Report.ReportID := StrToInt(ReportIdEdit.Text);

  while (Length(SendStr) > 0) and OK do
  begin
    for Cnt := 0 to 63 do
      if (Cnt < Length(SendStr)) then Report.Data[Cnt] := ORD(SendStr[Cnt + 1])
      else Report.Data[Cnt] := 0;

    NumBytes := 0;
    if sender = SendButton then NumBytes := HIDWriteDevice(Report);
    if sender = SetReportButton then NumBytes := HIDSetOutputReport(Report);
    if sender = SetFeatureButton then NumBytes := HIDSetFeatureReport(Report);

    Ok := NumBytes > 0;

    Delete(SendStr, 1, NumBytes);
    Application.ProcessMessages;
  end;

  if not OK
    then OpenIndicator.Brush.Color := clred
  else OpenIndicator.Brush.Color := clLime;
end;

procedure THIDtestForm.ClearRecButtonClick(Sender: TObject);
begin
  InputWindow.Lines.Clear;
end;

procedure THIDtestForm.ClearSendButtonClick(Sender: TObject);
begin
  OutputWindow.Lines.Clear;
end;

procedure THIDtestForm.FormCreate(Sender: TObject);
begin
  IsOpen := false;
  UpdateKeys;
  HIDsetEventHandler(@HIDreader); // assign callback procedure for HID reader
  USBsetEventHandler(@CheckUSBevent); // assign callback procedure for USB change event
end;

procedure THIDtestForm.CloseButtonClick(Sender: TObject);
begin
  if IsOpen then HIDcloseDevice;
  IsOpen := False;
  CurrentDevice := -1;
  OpenIndicator.Brush.Color := clBtnFace;
  Updatekeys;
end;

procedure THIDtestForm.FormDestroy(Sender: TObject);
begin
  if IsOpen then HIDCloseDevice;
end;

initialization
  NumDevices := 0;
  CurrentDevice := -1;
  ProgDir := ExtractFilepath(Application.Exename);
  if ProgDir[Length(ProgDir)] <> '\'
    then ProgDir := ProgDir + '\';
  ChDir(ProgDir);
end.

